process.env.NODE_ENV = 'test';

const chai = require('chai');

const dirtyChai = require('dirty-chai');

chai.use(dirtyChai);

const chaiXml = require('chai-xml');

chai.use(chaiXml);

const chaiHttp = require('chai-http');

const assert = chai.assert;

const expect = chai.expect;

const should = chai.should(); // actually call the function

const libs = `${process.cwd()}/libs`;

const server = require(`${libs}/app`);

chai.use(chaiHttp);

describe('API is running', () => {
    it('should respond it\'s running', (done) => {
        chai.request(server)
            .get('/api/v1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('msg');
                done();
            })
    });
});

describe('request enrollments', () => {
    it('should list the year range', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment/year_range')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('start_year');
                res.body.result[0].should.have.property('end_year');
                done();
            });
    });

    it('should list the education level', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment/education_level')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the administrative dependencies', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment/adm_dependency ')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list enrollments', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });

    it('should list enrollments with valid filters', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?filter=min_year:2014,state:41')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });

    it('should list enrollments with invalid filters', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?filter=foo:2010,bar:41')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });

    it('should list enrollments with valid dimensions', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?dims=region,state,adm_dependency,location&filter=min_year:2014,region:4')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('region_name');
                res.body.result[0].should.have.property('state_name');
                res.body.result[0].should.have.property('adm_dependency_name');
                res.body.result[0].should.have.property('location_name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });

    it('should list enrollments with invalid dimensions', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?dims=foo,bar')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });

    it('should list enrollments with valid dimensions and filters', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?dims=region,state,education_level,school&filter=min_year:2013,max_year:2014,city:3287')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('region_name');
                res.body.result[0].should.have.property('state_name');
                res.body.result[0].should.have.property('school_name');
                res.body.result[0].should.have.property('education_level');
                res.body.result[0].should.have.property('total');
                res.body.result[0].should.have.property('year');
                done();
            });
    });

    it('should list enrollments using all dimensions and filters', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment?dims=region,state,city,education_level,school,adm_dependency,location&filter=min_year:2013,max_year:2014,city:3287,adm_dependency:3,location:1,education_level:99')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('region_name');
                res.body.result[0].should.have.property('state_name');
                res.body.result[0].should.have.property('school_name');
                res.body.result[0].should.have.property('education_level');
                res.body.result[0].should.have.property('location_name');
                res.body.result[0].should.have.property('adm_dependency_name');
                res.body.result[0].should.have.property('total');
                res.body.result[0].should.have.property('year');
                done();
            });
    });


});

describe('request regions', () => {
    it('should list all regions', (done) => {
        chai.request(server)
            .get('/api/v1/region')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list region by id', (done) => {
        chai.request(server)
            .get('/api/v1/region/1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result.should.have.length(1);
                res.body.result[0].should.have.property('pk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });
});

describe('request states', () => {
    it('should list all states', (done) => {
        chai.request(server)
            .get('/api/v1/state')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list a state by id', (done) => {
        chai.request(server)
            .get('/api/v1/state/11')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result.should.have.length(1);
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list states by region id', (done) => {
        chai.request(server)
            .get('/api/v1/state/region/1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });
});

describe('request cities', () => {
    it('should list all cities', (done) => {
        chai.request(server)
            .get('/api/v1/city')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            });
    });

    it('should list a city by id', (done) => {
        chai.request(server)
            .get('/api/v1/city/4102802')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            });
    });

    it('should list a city by codigo_ibge', (done) => {
        chai.request(server)
            .get('/api/v1/city/ibge/1200013')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list all cities from a state', (done) => {
        chai.request(server)
            .get('/api/v1/city/state/41')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            })
    })
});

describe('request schools', () => {
    it('should list a school by id', (done) => {
        chai.request(server)
            .get('/api/v1/school/11000023')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_escola_id');
                res.body.result[0].should.have.property('ano_censo');
                res.body.result[0].should.have.property('cod_entidade');
                res.body.result[0].should.have.property('fk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                done();
            });
    });

    it('should list all schools from a state', (done) => {
        chai.request(server)
            .get('/api/v1/school/state/41')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_escola_id');
                res.body.result[0].should.have.property('ano_censo');
                res.body.result[0].should.have.property('cod_entidade');
                res.body.result[0].should.have.property('fk_municipio_id');
                done();
            });
    });

    it('should list all schools from a city', (done) => {
        chai.request(server)
            .get('/api/v1/school/city/4102802')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_escola_id');
                res.body.result[0].should.have.property('ano_censo');
                res.body.result[0].should.have.property('cod_entidade');
                res.body.result[0].should.have.property('fk_estado_id');
                done();
            })
    })
});

describe('test response', () => {
    it('should list all regions in json', (done) => {
        chai.request(server)
            .get('/api/v1/region')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                done();
            });
    });

    it('should list all regions in xml', (done) => {
        chai.request(server)
            .get('/api/v1/region?format=xml')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.xml;
                done();
            });
    });

    it('should list all regions in csv', (done) => {
        chai.request(server)
            .get('/api/v1/region?format=csv')
            .end((err, res) => {
                res.should.have.status(200);
                done();
            });
    });
});
